//& *** (c) 2006-2011 The HPGCC3 Team ***
//& Claudio Lapilli
//& Ingo Blank
//&
//& This file is licensed under the terms and conditions of the
//& HPGCC3 license that is included with the source distribution.
//& *** (c) 2006-2011 The HPGCC3 Team ***



// $Id: sat3_skipob.c 572 2008-03-16 23:20:33Z claudio $

#include <hpgcc3.h>
#include <master_globals.h>


extern SAT_OBJECT sat3_findfirst(SAT_OBJECT dir);
extern SAT_OBJECT __sat3_dirskiplibs(SAT_OBJECT objaddr);


#define STBL(prolog,size) (prolog + (size << 20))

#define SAT_TOTALOBJECTS 34

// TABLE TO SKIP OBJECTS
const unsigned int const __skipobtbl[SAT_TOTALOBJECTS]={

// FIXED-SIZE OBJECTS
    STBL(SAT_DOBINT,10),
    STBL(SAT_DOREAL,21),
    STBL(SAT_DOEREL,26),
    STBL(SAT_DOCMP,37),
    STBL(SAT_DOECMP,47),
    STBL(SAT_DOCHAR,7),
    STBL(SAT_DOROMP,11),
    STBL(SAT_DOFLASHP,12),

// LIST-LIKE OBJECTS
    STBL(SAT_DOLIST,0xFF),
    STBL(SAT_DOCOL,0xFF),
    STBL(SAT_DOEXT,0xFF),
    STBL(SAT_DOMATRIX,0xFF),
    STBL(SAT_DOSYMB,0xFF),

// IDNT-LIKE OBJECTS
    STBL(SAT_DOIDNT,0xFE),
    STBL(SAT_DOLAM,0xFE),

// SIZE-SKIPPABLE
    STBL(SAT_DOARRY,0xFD),
    STBL(SAT_DOBAK,0xFD),
    STBL(SAT_DOCODE,0xFD),
    STBL(SAT_DOCSTR,0xFD),
    STBL(SAT_DOGROB,0xFD),
    STBL(SAT_DOHXS,0xFD),
    STBL(SAT_DOINT,0xFD),
    STBL(SAT_DOLNKARRY,0xFD),
    STBL(SAT_DOLIB,0xFD),
    STBL(SAT_DOEXT0,0xFD),
    STBL(SAT_DOEXT1,0xFD),        // COULD BE WRONG - ACPTR IS PROBABLY A FIXED-SIZE OBJECT
    STBL(SAT_DOEXT2,0xFD),
    STBL(SAT_DOEXT3,0xFD),
    STBL(SAT_DOEXT4,0xFD),
    STBL(SAT_DOAPLET,0xFD),    // MIGHT BE WRONG

// SPECIAL OBJECTS
    STBL(SAT_DORRP,0xFC),
    STBL(SAT_DOTAG,0xFB),
    STBL(SAT_DOLNGCOMP,0xFA),  
    STBL(SAT_DOLNGREAL,0xF9),  

};



SAT_OBJECT sat3_skipob(SAT_OBJECT obaddr)
{
    unsigned int const *tbl=__skipobtbl;
//    int item;
    register int f,prolog,value;
    register SAT_OBJECT diroff;

	if(obaddr==0) return 0;
	
    prolog=sat3_peek(obaddr,5);

    for(f=0;f<SAT_TOTALOBJECTS;++f)
    
        if( ((value=tbl[f])&0xfffff)==prolog ) {
    
            value>>=20;
            switch(value)
            {
            case 0xf9:
            	// LONG REAL
            	obaddr+=5;
            	obaddr+=sat3_peek(obaddr,5);	// SKIP MANT
            	obaddr+=sat3_peek(obaddr,5);	// SKIP EXP
            	return obaddr;
            	
            case 0xfa:
            	// LONG COMPLEX
            	obaddr+=5;
            	obaddr+=sat3_peek(obaddr,5);	// SKIP MANT
            	obaddr+=sat3_peek(obaddr,5);	// SKIP EXP
            	obaddr+=sat3_peek(obaddr,5);	// SKIP MANT
            	obaddr+=sat3_peek(obaddr,5);	// SKIP EXP
            	return obaddr;

            case 0xfb:
                // DOTAG OBJECTS
                return sat3_skipob(sat3_sknameup(obaddr+5)-2);
    
            case 0xfc:
                // DIRECTORIES
                diroff=__sat3_dirskiplibs(obaddr);
                if(!diroff) return 0;
                if(sat3_peek(diroff,5)==0) return diroff+5;		// CHECK FOR EMPTY DIRS
                obaddr=sat3_findfirst(obaddr);
                return sat3_skipob(obaddr);
    
            case 0xfd:
                // SIZE-SKIPPABLE OBJECTS
                obaddr+=5;
                return obaddr+sat3_peek(obaddr,5);
    
            case 0xfe:
                // IDNT-LIKE OBJECTS
                obaddr+=5;
                return obaddr+((sat3_peek(obaddr,2)+1)<<1);
    
            case 0xff:
                // LIST-LIKE OBJECTS
                obaddr+=5;
                while(sat3_peek(obaddr,5)!=SAT_SEMI)
                {
                    obaddr=sat3_skipob(obaddr);
                }
                return obaddr+5;
    
            default:
                return obaddr+value;
            }
        }
    

    // SKIP ROM POINTERS
    return obaddr+5;

}

int sat3_sizeof(SAT_OBJECT obj)
{
return (int)(sat3_skipob(obj)-obj);
}

// RETURN SIZE OF A SATURN TYPE IN NIBBLES, 0 FOR UNKNOWN OR VARIABLE SIZE OBJECTS

int sat3_typesize(int prologtype)
{
    unsigned int const *tbl=__skipobtbl;
//    int item;
    register int f,value;

    for(f=0;f<SAT_TOTALOBJECTS;++f) {
    
        if( ((value=tbl[f])&0xfffff)==prologtype ) {
    
            value>>=20;
			if(value>0xf0) return 0;
			return value;
        }
    
	}

return 0;

}

int sat3_strsize(SAT_OBJECT obj)
{
	
	if(sat3_isptr(obj)) obj=sat3_peek(obj,5);
	
	int prologtype=sat3_peek(obj,5);
	
    unsigned int const *tbl=__skipobtbl;
//    int item;
    register int f,value;

    for(f=0;f<SAT_TOTALOBJECTS;++f) {
    
        if( ((value=tbl[f])&0xfffff)==prologtype ) {
    
            value>>=20;
			if(value!=0xfd) return 0;	// NOT A STRING OBJECT
			return sat3_peek(obj+5,5)-5;
        }
    
	}

return 0;

}

