//& *** (c) 2006-2011 The HPGCC3 Team ***
//& Claudio Lapilli
//& Ingo Blank
//&
//& This file is licensed under the terms and conditions of the
//& HPGCC3 license that is included with the source distribution.
//& *** (c) 2006-2011 The HPGCC3 Team ***

#include <hpgcc3.h>
#include <master_globals.h>


extern unsigned int __startup_options;
typedef int (*main_func)(void);

extern void __exception_install(unsigned int *);
extern void __exception_uninst(unsigned int *);
extern int _exit_save(unsigned int *);
extern void __exit_cleanup(void);
extern void __keyb_init(unsigned int *);
extern void __keyb_stop(unsigned int *);
extern void __keyb_waitrelease();
extern void __console_init(int);

void __launch_startup(main_func main_ptr,unsigned int *org_regs)
{

    // DECLARE LOCALLY TO REDUCE EXECUTABLE SIZE
    unsigned int state_buffer[16],lcd_buffer[17],clk_buffer[8],tmr_buffer[13],int_buffer[11],keyb_buffer[2];
    unsigned int org_handler[8];
    


    if(!(__startup_options&_ENABLE_INTS))   {
    	 cpu_intsave(int_buffer);
    	 cpu_intoff();
    }

	state_buffer[14]=(unsigned int)org_regs;		// SAVE ORIGINAL ARM REGISTERS FOR USE BY THE SYSTEM
	state_buffer[15]=(unsigned int)&org_handler;		// SAVE ORIGINAL ARM REGISTERS FOR USE BY THE SYSTEM

    __mg->__exit_stk_state=state_buffer;

    // SAVE HARDWARE STATE
   if(!(__startup_options&_NO_SCREEN)) lcd_save(lcd_buffer);

   if((__startup_options&_CLK_MODE_MSK)!=_NO_SPEEDCHANGE) {
   cpu_clksave(clk_buffer);
   
   switch((__startup_options&_CLK_MODE_MSK)>>4)
   {
   	case 1:
   	__cpu_setspeed(CLK_1MHZ);
   	break;
   	case 2:
   	__cpu_setspeed(CLK_6MHZ);
   	break;
   	case 3:
   	__cpu_setspeed(CLK_48MHZ);
   	break;
   	case 4:
   	__cpu_setspeed(CLK_75MHZ);
   	break;
   	case 5:
   	__cpu_setspeed(CLK_120MHZ);
   	break;
   	case 6:
   	__cpu_setspeed(CLK_152MHZ);
   	break;
   	case 7:
   	__cpu_setspeed(CLK_192MHZ);
   	break;
   	case 0:
   	default:
   	__cpu_setspeed(CLK_12MHZ);
   	break;
   }   	
   }
   
    
	if(!(__startup_options&_NO_EXCEPTIONS)) {
   
   // START EXCEPTION HANDLERS AND IRQ MACHINERY
    __exception_install(org_handler);
	}

   if(!(__startup_options&_NO_TIMERS)) {
   	tmr_save(tmr_buffer);
   	tmr_setup();
   }

   if(!(__startup_options&_NO_KEYBOARD)) {
   	__keyb_init(keyb_buffer);
   }


// ***INGO: ADD YOUR INITIALIZATION HOOK HERE ***
	if(!(__startup_options&_NO_SCREEN)) {

		if((__startup_options&_NO_GUI_CONSOLE)) __console_init(1);	// INITIALIZE STANDARD CONSOLE
		else __console_init(2);	//INITIALIZE GRAYSCALE CONSOLE
	}
	else __console_init(0); // INITIALIZE NULL CONSOLE (SIMILAR TO /dev/null)

    
    // WILL RETURN 0 WHEN EXITTING
   if(_exit_save((unsigned int *)state_buffer)) 
    {
    (*main_ptr)(); 
    __exit_cleanup(); 
    }


   if(!(__startup_options&_NO_KEYBOARD)) {
		if(!(__startup_options&_NO_WAITKEY)) __keyb_waitrelease();
	   __keyb_stop(keyb_buffer);
   }


	if(!(__startup_options&_NO_TIMERS)) tmr_restore(tmr_buffer);
	
	if(!(__startup_options&_NO_EXCEPTIONS)) {
  	// UNINSTALL EXCEPTION HANDLERS
  	// ALSO REMOVE IRQ HANDLERS
	__exception_uninst(org_handler);
	}
	
	if((__startup_options&_NO_SCREEN)) {
	   if((__startup_options&_CLK_MODE_MSK)!=_NO_SPEEDCHANGE) { lcd_sync();}
	   }
   if((__startup_options&_CLK_MODE_MSK)!=_NO_SPEEDCHANGE) cpu_clkrestore(clk_buffer);
   if(!(__startup_options&_NO_SCREEN)) lcd_restore(lcd_buffer);
   else {
   if((__startup_options&_CLK_MODE_MSK)!=_NO_SPEEDCHANGE) { __lcd_fix(); }
   }
   
   
   
    if(!(__startup_options&_ENABLE_INTS)) {
    	cpu_intrestore(int_buffer);
    }


}
